@@ -8,18 +8,21 @@ need to be stored, and as a class with a __call__ method if there are parameters
 import copy
 import logging
 import re
-from .qualtrim import quality_trim_index, nextseq_trim_index
-from .align import Aligner, SEMIGLOBAL, InsertAligner
-from .util import complement, reverse_complement, mean
+from atropos.qualtrim import quality_trim_index, nextseq_trim_index
+from atropos.align import Aligner, InsertAligner, SEMIGLOBAL, START_WITHIN_SEQ1, STOP_WITHIN_SEQ2
+from atropos.util import base_complements, reverse_complement, mean, quals2ints
 
 # Base classes
 
 class ReadPairModifier(object):
-    """Superclass of modifiers that edit a pair of reads simultaneously."""
+    """Base class of modifiers that edit a pair of reads simultaneously.
+    """
     def __call__(self, read1, read2):
         raise NotImplemented()
 
 class Trimmer(object):
+    """Base class of modifiers that trim bases from reads.
+    """
     def __init__(self):
         self.trimmed_bases = 0
     
@@ -144,45 +147,179 @@ class AdapterCutter(object):
         self.with_adapters += 1
         return trimmed_read
 
-class InsertAdapterCutter(ReadPairModifier):
+class ErrorCorrectorMixin(object):
+    """Provides a method for error correction.
+    
+    Args:
+        mismatch_action: The action to take when a mismatch between the
+            overlapping portions of read1 and read2 is encountered. Valid
+            values are 'liberal', 'conservative', 'N'.
+        min_qual_difference: When mismatch_action=='conservative', the minimum
+            difference in base quality between read1 and read2 required to
+            perform the correction.
+    """
+    def __init__(self, mismatch_action=None, min_qual_difference=1):
+        self.mismatch_action = mismatch_action
+        self.r1r2_min_qual_difference = min_qual_difference
+        self.r2r1_min_qual_difference = -1 * min_qual_difference
+        self.corrected_pairs = 0
+        self.corrected_bp = [0, 0]
+    
+    def correct_errors(self, read1, read2, insert_match):
+        # Do not attempt to correct an already corrected read
+        if read1.corrected > 0 or read2.corrected > 0:
+            return
+        
+        # read2 reverse-complement is the reference, read1 is the query
+        r1_seq = list(read1.sequence)
+        r2_seq = list(read2.sequence)
+        l2 = len(r2_seq)
+        
+        has_quals = read1.qualities and read2.qualities
+        if has_quals:
+            r1_qual = list(read1.qualities)
+            r2_qual = list(read2.qualities)
+        elif self.mismatch_action in ('liberal', 'conservative'):
+            raise Exception("Cannot perform quality-based error correction on "
+                            "reads lacking quality information")
+        
+        r1_start = insert_match[2]
+        r1_end = insert_match[3]
+        r1_changed = 0
+        r2_start = l2 - insert_match[1]
+        r2_end = l2 - insert_match[0]
+        r2_changed = 0
+        quals_equal = []
+        
+        for i, j in zip(range(r1_start, r1_end), range(r2_end - 1, r2_start, -1)):
+            b1 = r1_seq[i]
+            b2 = base_complements[r2_seq[j]]
+            if b1 == b2:
+                continue
+            if self.mismatch_action == 'N':
+                r1_seq[i] = 'N'
+                r2_seq[j] = 'N'
+                r1_changed += 1
+                r2_changed += 1
+            elif b1 == 'N':
+                r1_seq[i] = b2
+                if has_quals:
+                    r1_qual[i] = r2_qual[j]
+                r1_changed += 1
+            elif b2 == 'N':
+                r2_seq[j] = base_complements[b1]
+                if has_quals:
+                    r2_qual[j] = r1_qual[i]
+                r2_changed += 1
+            elif has_quals:
+                diff = ord(r1_qual[i]) - ord(r2_qual[j])
+                if diff >= self.r1r2_min_qual_difference:
+                    r2_seq[j] = base_complements[b1]
+                    r2_qual[j] = r1_qual[i]
+                    r2_changed += 1
+                elif diff <= self.r2r1_min_qual_difference:
+                    r1_seq[i] = b2
+                    r1_qual[i] = r2_qual[j]
+                    r1_changed += 1
+                elif self.mismatch_action == 'liberal':
+                    quals_equal.append((i, j, b1, b2))
+        
+        if quals_equal:
+            mean_qual1 = mean([ord(b) for b in r1_qual[r1_start:r1_end]])
+            mean_qual2 = mean([ord(b) for b in r2_qual[r2_start:r2_end]])
+            # Only make the corrections if one read is significantly better
+            # than the other.
+            # TODO: this method of determining whether one read is better
+            # than the other is crude - come up with something better.
+            diff = mean_qual1 - mean_qual2
+            if diff > 1:
+                # read1 is better than read2
+                for i, j, b1, b2 in quals_equal:
+                    r2_seq[j] = base_complements[b1]
+                    r2_qual[j] = r1_qual[i]
+                    r2_changed += 1
+            elif diff < -1:
+                # read2 is better than read1
+                for i, j, b1, b2 in quals_equal:
+                    r1_seq[i] = b2
+                    r1_qual[i] = r2_qual[j]
+                    r1_changed += 1
+        
+        if r1_changed or r2_changed:
+            self.corrected_pairs += 1
+            if r1_changed:
+                self.corrected_bp[0] += r1_changed
+                read1.sequence = ''.join(r1_seq)
+                read1.corrected = r1_changed
+                if has_quals:
+                    read1.qualities = ''.join(r1_qual)
+            if r2_changed:
+                self.corrected_bp[1] += r2_changed
+                read2.sequence = ''.join(r2_seq)
+                read2.corrected = r2_changed
+                if has_quals:
+                    read2.qualities = ''.join(r2_qual)
+
+class InsertAdapterCutter(ReadPairModifier, ErrorCorrectorMixin):
     """
     AdapterCutter that uses InsertAligner to first try to identify
     insert overlap before falling back to semi-global adapter alignment.
     """
     def __init__(self, adapter1, adapter2, action='trim', mismatch_action=None,
                  symmetric=True, min_insert_overlap=1, **aligner_args):
+        ErrorCorrectorMixin.__init__(self, mismatch_action)
         self.adapter1 = adapter1
         self.adapter2 = adapter2
         self.aligner = InsertAligner(adapter1.sequence, adapter2.sequence,
             min_insert_overlap=min_insert_overlap, **aligner_args)
         self.min_insert_len = min_insert_overlap
         self.action = action
-        self.mismatch_action = mismatch_action
         self.symmetric = symmetric
         self.with_adapters = [0, 0]
-        self.corrected_pairs = 0
-        self.corrected_bp = [0, 0]
     
     def __call__(self, read1, read2):
-        if len(read1) < self.min_insert_len or len(read2) < self.min_insert_len:
+        read_lengths = [len(r) for r in (read1, read2)]
+        if any(l < self.min_insert_len for l in read_lengths):
             return (read1, read2)
         
         match = self.aligner.match_insert(read1.sequence, read2.sequence)
+        read1.insert_overlap = read2.insert_overlap = (match is not None)
+        insert_match = None
+        correct_errors = False
         
         if match:
             insert_match, adapter_match1, adapter_match2 = match
-            if self.mismatch_action and insert_match[5] > 0:
-                self.correct_errors(read1, read2, insert_match)
+            correct_errors = self.mismatch_action and insert_match[5] > 0
         else:
             adapter_match1 = self.adapter1.match_to(read1)
             adapter_match2 = self.adapter2.match_to(read2)
+            # If the adapter matches are complementary, perform error correction
+            if (self.mismatch_action and adapter_match1 and adapter_match2 and
+                        adapter_match1.rstart == adapter_match2.rstart):
+                    insert_match = (
+                        read_lengths[1] - adapter_match1.rstart,
+                        read_lengths[1], 0, adapter_match1.rstart)
+                    correct_errors = True
         
-        if self.symmetric:
-            if adapter_match1 is None and adapter_match2:
-                adapter_match1 = adapter_match2.copy()
-            if adapter_match2 is None and adapter_match1:
+        # If exactly one of the two alignments failed and symmetrix is True,
+        # duplicate the good alignment
+        if self.symmetric and sum(
+                bool(m) for m in (adapter_match1, adapter_match2)) == 1:
+            if adapter_match1:
                 adapter_match2 = adapter_match1.copy()
-            
+            else:
+                adapter_match1 = adapter_match2.copy()
+            if self.mismatch_action and not insert_match:
+                # Assume that the symmetric read segments overlap and
+                # perform error correction
+                insert_match = (
+                    read_lengths[1] - adapter_match1.rstart,
+                    read_lengths[1], 0, adapter_match1.rstart)
+                correct_errors = True
+        
+        if correct_errors:
+            self.correct_errors(read1, read2, insert_match)
+        
         return (
             self.trim(read1, self.adapter1, adapter_match1, 0),
             self.trim(read2, self.adapter2, adapter_match2, 1)
@@ -211,110 +348,75 @@ class InsertAdapterCutter(ReadPairModifier):
                 # set masked sequence as sequence with original quality
                 trimmed_read.sequence = masked_sequence
                 trimmed_read.qualities = read.qualities
+            elif self.action == 'lower':
+                # TODO: offer option to mask with lower-case of trimmed base
+                # This will happen as part of the refactoring to modify
+                # Sequences in-place.
+                pass
         
         trimmed_read.match = match
         trimmed_read.match_info = [match.get_info_record()]
         
         self.with_adapters[read_idx] += 1
         return trimmed_read
+
+class OverwriteRead(ReadPairModifier):
+    """If one read is of significantly worse quality than the other, overwrite
+    the poor quality read with the good-quality read.
     
-    def correct_errors(self, read1, read2, insert_match):
-        # read2 reverse-complement is the reference, read1 is the query
-        r1_seq = list(read1.sequence)
-        r2_seq = list(read2.sequence)
-        l2 = len(r2_seq)
+    Computes the mean quality over the first `window_size` bases. If one read
+    has mean quality < `worse_read_min_quality` and the other read has mean
+    quality >= `better_read_min_quality`, replace the entire worse read with
+    the better read.
+    
+    Args:
+        worse_read_min_quality: The minimum quality below which a read is
+            considered 'poor'
+        better_read_min_quality: The quality above which a read is considered
+            'good'
+        window_size: The number of bases over which to compute mean quality
+        base: The base for ascii-to-int quality conversion
+        summary_fn: Function to summarize base quality (default=mean)
+    """
+    def __init__(self, worse_read_min_quality, better_read_min_quality,
+                 window_size, base=33, summary_fn=mean):
+        self.worse_read_min_quality = worse_read_min_quality
+        self.better_read_min_quality = better_read_min_quality
+        self.window_size = window_size
+        self.base = base
+        self.summary_fn = summary_fn
+    
+    def __call__(self, read1, read2):
+        if len(read1) < self.window_size or len(read2) < self.window_size:
+            return (read1, read2)
+        if not (read1.qualities and read2.qualities):
+            raise Exception("OverwriteRead modifier does not work with reads "
+                            "lacking base qualities.")
+        q1 = list(quals2ints(read1.qualities[:self.window_size], self.base))
+        s1 = self.summary_fn(q1)
         
-        has_quals = read1.qualities and read2.qualities
-        if has_quals:
-            r1_qual = list(read1.qualities)
-            r2_qual = list(read2.qualities)
-        elif self.mismatch_action in ('liberal', 'conservative'):
-            raise Exception("Cannot perform quality-based error correction on reads lacking quality information")
+        q2 = list(quals2ints(read2.qualities[:self.window_size], self.base))
+        s2 = self.summary_fn(q2)
         
-        r1_start = insert_match[2]
-        r1_end = insert_match[3]
-        r1_changed = 0
-        r2_start = l2 - insert_match[1]
-        r2_end = l2 - insert_match[0]
-        r2_changed = 0
-        quals_equal = []
+        if s1 < self.worse_read_min_quality and s2 >= self.better_read_min_quality:
+            # TODO: not sure what the right value is here
+            read2.corrected = 1
+            read1 = read2.reverse_complement()
+        elif s2 < self.worse_read_min_quality and s1 >= self.better_read_min_quality:
+            read1.corrected = 1
+            read2 = read1.reverse_complement()
         
-        for i, j in zip(range(r1_start, r1_end), range(r2_end - 1, r2_start, -1)):
-            b1 = r1_seq[i]
-            b2 = complement[r2_seq[j]]
-            if b1 == b2:
-                continue
-            if self.mismatch_action == 'N':
-                r1_seq[i] = 'N'
-                r2_seq[j] = 'N'
-                r1_changed += 1
-                r2_changed += 1
-            elif b1 == 'N':
-                r1_seq[i] = b2
-                if has_quals:
-                    r1_qual[i] = r2_qual[j]
-                r1_changed += 1
-            elif b2 == 'N':
-                r2_seq[j] = complement[b1]
-                if has_quals:
-                    r2_qual[j] = r1_qual[i]
-                r2_changed += 1
-            elif has_quals:
-                if r1_qual[i] > r2_qual[j]:
-                    r2_seq[j] = complement[b1]
-                    r2_qual[j] = r1_qual[i]
-                    r2_changed += 1
-                elif r2_qual[j] > r1_qual[i]:
-                    r1_seq[i] = b2
-                    r1_qual[i] = r2_qual[j]
-                    r1_changed += 1
-                elif self.mismatch_action == 'liberal':
-                    quals_equal.append((i, j, b1, b2))
-        
-        if quals_equal:
-            mean_qual1 = mean([ord(b) for b in r1_qual[r1_start:r1_end]])
-            mean_qual2 = mean([ord(b) for b in r2_qual[r2_start:r2_end]])
-            # Only make the corrections if one read is significantly better
-            # than the other.
-            # TODO: this method of determining whether one read is better
-            # than the other is crude - come up with something better.
-            diff = mean_qual1 - mean_qual2
-            if diff > 1:
-                # read1 is better than read2
-                for i, j, b1, b2 in quals_equal:
-                    r2_seq[j] = complement[b1]
-                    r2_qual[j] = r1_qual[i]
-                    r2_changed += 1
-            elif diff < -1:
-                # read2 is better than read1
-                for i, j, b1, b2 in quals_equal:
-                    r1_seq[i] = b2
-                    r1_qual[i] = r2_qual[j]
-                    r1_changed += 1
-        
-        if r1_changed or r2_changed:
-            self.corrected_pairs += 1
-            if r1_changed:
-                self.corrected_bp[0] += r1_changed
-                read1.sequence = ''.join(r1_seq)
-                read1.corrected = r1_changed
-                if has_quals:
-                    read1.qualities = ''.join(r1_qual)
-            if r2_changed:
-                self.corrected_bp[1] += r2_changed
-                read2.sequence = ''.join(r2_seq)
-                read2.corrected = r2_changed
-                if has_quals:
-                    read2.qualities = ''.join(r2_qual)
+        return (read1, read2)
 
 class UnconditionalCutter(Trimmer):
-    """
-    A modifier that unconditionally removes the first n or the last n bases from a read.
-    Equivalent to MinCutter with count_trimmed=False and only_trimmed=False, but
-    UnconditionalCutter is applied before adapter trimming.
+    """A modifier that unconditionally removes the first n or the last n bases
+    from a read. Equivalent to MinCutter with count_trimmed=False and
+    only_trimmed=False, but UnconditionalCutter is applied before adapter
+    trimming.
     
-    If the length is positive, the bases are removed from the beginning of the read.
-    If the length is negative, the bases are removed from the end of the read.
+    If the length is positive, the bases are removed from the beginning of the
+    read. If the length is negative, the bases are removed from the end of the
+    read.
     """
     
     display_str = "Cut unconditionally"
@@ -328,10 +430,10 @@ class UnconditionalCutter(Trimmer):
         return self.clip(read, self.front_length, self.back_length)
 
 class MinCutter(Trimmer):
-    """
-    Ensure that a minimum number of bases have been trimmed off each end.
-    count_trimmed :: whether to consider bases cut before or during adapter trimming
-        when counting the number of bases that have already been cut
+    """Ensure that a minimum number of bases have been trimmed off each end.
+    
+    count_trimmed :: whether to consider bases cut before or during adapter
+        trimming when counting the number of bases that have already been cut
     only_trimmed :: only cut read ends if they have already been adapter-trimmed.
     """
     
@@ -364,7 +466,10 @@ class MinCutter(Trimmer):
             if self.count_trimmed:
                 trimmed = read.clipped[offset] + read.clipped[offset+2]
                 if read.match:
-                    trimmed += sum(i.rsize_total for i in read.match_info if is_front == i.is_front)
+                    trimmed += sum(
+                        i.rsize_total
+                        for i in read.match_info
+                        if is_front == i.is_front)
             elif read.match:
                 trimmed = read.clipped[offset+2]
             else:
@@ -392,7 +497,8 @@ class LengthTagModifier(object):
     def __call__(self, read):
         read = read[:]
         if read.name.find(self.length_tag) >= 0:
-            read.name = self.regex.sub(self.length_tag + str(len(read.sequence)), read.name)
+            read.name = self.regex.sub(
+                self.length_tag + str(len(read.sequence)), read.name)
         return read
 
 class SuffixRemover(object):
@@ -583,54 +689,65 @@ class SwiftBisulfiteTrimmer(ReadPairModifier):
 # TODO: InsertAdapterCutter should save the insert match, and
 # MergeOverlapping should use that rather than doing another alignment
 
-class MergeOverlapping(ReadPairModifier):
-    def __init__(self, min_overlap=0.9, error_rate=0.1):
+class MergeOverlapping(ReadPairModifier, ErrorCorrectorMixin):
+    def __init__(self, min_overlap=0.9, error_rate=0.1, mismatch_action=None):
+        ErrorCorrectorMixin.__init__(self, mismatch_action)
         self.min_overlap = int(min_overlap) if min_overlap > 1 else min_overlap
         self.error_rate = error_rate
     
     def __call__(self, read1, read2):
-        if read1.match and not read1.match.front and read2.match and not read2.match.front:
-            l1 = len(read1.sequence)
-            l2 = len(read2.sequence)
-            min_overlap = self.min_overlap
-            if min_overlap <= 1:
-                min_overlap = max(2, round(self.min_overlap * min(l1, l2)))
-            
-            if l1 < min_overlap or l2 < min_overlap:
-                return (read1, read2)
-            
-            aligner = Aligner(read1.sequence, self.error_rate, SEMIGLOBAL)
-            read2_rc = reverse_complement(read2.sequence)
-            # TODO: Resolve overlap. We offer the user the choice to set mismatching bases to
-            # the one with the highest quality or to 'N'. Additionally, set qualities of overlapping
-            # bases to the highest value. The acutal alignment is not returned, so if there are
-            # mismatches we don't know what they are, and figuring it out is complicated by indels.
-            alignment = aligner.locate(read2_rc)
-            if alignment:
-                r1_start, r1_stop, r2_start, r2_stop, matches, errors = alignment
-                if matches >= min_overlap:
-                    if r2_start == 0 and r2_stop == l2:
-                        # r2 is fully contained in r1
-                        pass
-                    elif r1_start == 0 and r1_stop == l1:
-                        # r1 is fully contained in r2
-                        read1.sequence = read2_rc
-                        read1.qualities = "".join(reversed(read2.qualities))
-                    elif r1_start > 0:
-                        read1.sequence += read2_rc[r2_stop:]
-                        if read1.qualities and read2.qualities:
-                            read1.qualities += "".join(reversed(read2.qualities))[r2_stop:]
-                    elif r2_start > 0:
-                        read1.sequence = read2_rc + read1[r1_stop:]
-                        if read1.qualities and read2.qualities:
-                            read1.qualities = "".join(reversed(read2.qualities)) + read1.qualities[r1_stop:]
-                    else:
-                        raise Exception(
-                            "Invalid alignment while trying to merge read {}: {}".format(
-                                read1.name, ",".join(str(i) for i in alignment)))
-                    
-                    read1.merged = True
-                    read2 = None
+        l1 = len(read1.sequence)
+        l2 = len(read2.sequence)
+        min_overlap = self.min_overlap
+        if min_overlap <= 1:
+            min_overlap = max(2, round(self.min_overlap * min(l1, l2)))
+        
+        if l1 < min_overlap or l2 < min_overlap:
+            return (read1, read2)
+        
+        insert_matched = read1.insert_overlap and read2.insert_overlap
+        
+        if insert_matched:
+            # If we've already determined that there is an insert overlap
+            # with a 3' overhang, we can constrain our alignment
+            aflags = START_WITHIN_SEQ1 | STOP_WITHIN_SEQ2
+        else:
+            aflags = SEMIGLOBAL
+        # align read1 to read2 reverse-complement to be compatible with InsertAligner
+        read2_rc = reverse_complement(read2.sequence)
+        aligner = Aligner(read2_rc, self.error_rate, aflags)
+        alignment = aligner.locate(read1.sequence)
+        
+        if alignment:
+            r2_start, r2_stop, r1_start, r1_stop, matches, errors = alignment
+            if matches >= min_overlap:
+                # Only correct errors if we haven't already done correction in the InsertAligner
+                if (self.mismatch_action and errors > 0 and not insert_matched and
+                        read1.corrected == 0 and read2.corrected == 0):
+                    self.correct_errors(read1, read2, alignment)
+                
+                if r2_start == 0 and r2_stop == l2:
+                    # r2 is fully contained in r1
+                    pass
+                elif r1_start == 0 and r1_stop == l1:
+                    # r1 is fully contained in r2
+                    read1.sequence = read2_rc
+                    read1.qualities = "".join(reversed(read2.qualities))
+                elif r1_start > 0:
+                    read1.sequence += read2_rc[r2_stop:]
+                    if read1.qualities and read2.qualities:
+                        read1.qualities += "".join(reversed(read2.qualities))[r2_stop:]
+                elif r2_start > 0:
+                    read1.sequence = read2_rc + read1.sequence[r1_stop:]
+                    if read1.qualities and read2.qualities:
+                        read1.qualities = "".join(reversed(read2.qualities)) + read1.qualities[r1_stop:]
+                else:
+                    raise Exception(
+                        "Invalid alignment while trying to merge read {}: {}".format(
+                            read1.name, ",".join(str(i) for i in alignment)))
+                
+                read1.merged = True
+                read2 = None
                 
         return (read1, read2)
 
@@ -642,7 +759,10 @@ class Modifiers(object):
     
     def add_modifier(self, mod_class, read=1|2, **kwargs):
         if issubclass(mod_class, ReadPairModifier):
-            assert self.paired == "both" and read == 1|2
+            if self.paired != "both" and read == 1|2:
+                raise ValueError(
+                    "Must have paired-end reads to use modifer {}".format(
+                    mod_class))
             mods = mod_class(**kwargs)
         else:
             mods = [None, None]
@@ -680,15 +800,23 @@ class Modifiers(object):
             mods = [self.modifiers[i] for i in self.modifier_indexes[mod_class]]
         else:
             mods = []
-        if mods and read:
-            mods = [m[read-1] for m in mods if m[read-1] is not None]
-        return mods
+        
+        if not (mods and read):
+            return mods
+        
+        read_mods = []
+        for m in mods:
+            if isinstance(m, ReadPairModifier):
+                read_mods.append(m)
+            elif m[read-1] is not None:
+                read_mods.append(m[read-1])
+        return read_mods
     
     def has_modifier(self, mod_class):
         return mod_class in self.modifier_indexes
     
     def get_adapters(self):
-        adapters = [None, None]
+        adapters = [[], []]
         if self.has_modifier(AdapterCutter):
             m1, m2 = self.get_modifiers(AdapterCutter)[0]
             if m1:
