@@ -13,8 +13,8 @@ from os.path import splitext
 import re
 import sys
 
-from .filters import NoFilter
-from .xopen import xopen, open_output
+from atropos.filters import NoFilter
+from atropos.xopen import STDOUT, STDERR, xopen, open_output
 
 def _shorten(s, n=100):
     """Shorten string s to at most n characters, appending "..." if necessary."""
@@ -40,16 +40,20 @@ class SequenceReader(object):
     """Read possibly compressed files containing sequences"""
     _close_on_exit = False
 
-    def __init__(self, file):
+    def __init__(self, file, mode='r'):
         """
         file is a path or a file-like object. In both cases, the file may
         be compressed (.gz, .bz2, .xz).
         """
         if isinstance(file, str):
-            file = xopen(file)
+            file = xopen(file, mode)
             self._close_on_exit = True
         self._file = file
-
+    
+    @property
+    def name(self):
+        return self._file.name
+    
     def close(self):
         if self._close_on_exit and self._file is not None:
             self._file.close()
@@ -69,7 +73,9 @@ except ImportError:
     pass
 
 class ColorspaceSequence(Sequence):
-    def __init__(self, name, sequence, qualities, primer=None, name2='', original_length=None, match=None, match_info=None, clipped=[0,0,0]):
+    def __init__(self, name, sequence, qualities, primer=None, name2='', original_length=None,
+                 match=None, match_info=None, clipped=[0,0,0], insert_overlap=False,
+                 merged=False, corrected=0):
         # In colorspace, the first character is the last nucleotide of the primer base
         # and the second character encodes the transition from the primer base to the
         # first real base of the read.
@@ -83,7 +89,9 @@ class ColorspaceSequence(Sequence):
             raise FormatError("In read named {0!r}: length of colorspace quality "
                 "sequence ({1}) and length of read ({2}) do not match (primer "
                 "is: {3!r})".format(rname, len(qualities), len(sequence), self.primer))
-        super(ColorspaceSequence, self).__init__(name, sequence, qualities, name2, original_length, match, match_info, clipped)
+        super(ColorspaceSequence, self).__init__(
+            name, sequence, qualities, name2, original_length, match, match_info, clipped,
+            insert_overlap, merged, corrected)
         if not self.primer in ('A', 'C', 'G', 'T'):
             raise FormatError("Primer base is {0!r} in read {1!r}, but it "
                 "should be one of A, C, G, T.".format(
@@ -106,7 +114,10 @@ class ColorspaceSequence(Sequence):
             self.original_length,
             self.match,
             self.match_info,
-            self.clipped)
+            self.clipped,
+            self.insert_overlap,
+            self.merged,
+            self.corrected)
 
 def sra_colorspace_sequence(name, sequence, qualities, name2):
     """Factory for an SRA colorspace sequence (which has one quality value too many)"""
@@ -344,6 +355,86 @@ class InterleavedSequenceReader(object):
     def __exit__(self, *args):
         self.close()
 
+class SAMReader(object):
+    """
+    Reader for SAM/BAM files. Paired-end files must be name-sorted. Does
+    not support secondary/supplementary reads.
+    """
+    def __init__(self, file, sequence_class=Sequence):
+        """
+        file is a filename or a file-like object.
+        If file is a filename, then .gz files are supported.
+        """
+        self._close_on_exit = False
+        if isinstance(file, str):
+            file = xopen(file, 'rb')
+            self._close_on_exit = True
+        self._file = file
+        self.sequence_class = sequence_class
+        self.delivers_qualities = True
+
+    def __iter__(self):
+        import pysam
+        return self._iter(pysam.AlignmentFile(self._file))
+    
+    def __enter__(self):
+        return self
+    
+    def __exit__(self, *args):
+        self.close()
+    
+    def close(self):
+        if self._close_on_exit and self._file is not None:
+            self._file.close()
+            self._file = None
+        
+    def _as_sequence(self, read):
+        return self.sequence_class(
+            read.query_name,
+            read.query_sequence,
+            ''.join(chr(33 + q) for q in read.query_qualities))
+
+class SingleEndSAMReader(SAMReader):
+    def _iter(self, sam):
+        for read in sam:
+            yield self._as_sequence(read)
+
+class Read1SingleEndSAMReader(SAMReader):
+    def _iter(self, sam):
+        for read in sam:
+            if read.is_read1:
+                yield self._as_sequence(read)
+
+class Read2SingleEndSAMReader(SAMReader):
+    def _iter(self, sam):
+        for read in sam:
+            if read.is_read2:
+                yield self._as_sequence(read)
+
+class PairedEndSAMReader(SAMReader):
+    def _iter(self, sam):
+        for reads in zip(sam, sam):
+            if reads[0].query_name != reads[1].query_name:
+                raise Exception(
+                    "Consecutive reads {}, {} in paired-end SAM/BAM file do not "
+                    "have the same name; make sure your file is name-sorted and "
+                    "does not contain any secondary/supplementary alignments.",
+                    reads[0].query_name, reads[1].query_name)
+            
+            if reads[0].is_read1:
+                assert reads[1].is_read2
+            else:
+                assert reads[1].is_read1
+                reads = (reads[1], reads[0])
+            
+            yield tuple(self._as_sequence(r) for r in reads)
+
+def paired_to_read1(reader):
+    for r1, r2 in reader: yield r1
+
+def paired_to_read2(reader):
+    for r1, r2 in reader: yield r2
+
 class BatchIterator(object):
     def __init__(self, reader, size, max_reads=None):
         self.reader = reader
@@ -395,7 +486,7 @@ class BatchIterator(object):
         self.reader.close()
 
 def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=None,
-                interleaved=False, qualities=None):
+                interleaved=False, qualities=None, single_input_read=None):
     """
     Open sequence files in FASTA or FASTQ format for reading. This is
     a factory that returns an instance of one of the ...Reader
@@ -414,8 +505,9 @@ def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=N
         are returned.
 
     fileformat -- If set to None, file format is autodetected from the file name
-        extension. Set to 'fasta', 'fastq', or 'sra-fastq' to not auto-detect.
-        Colorspace is not auto-detected and must always be requested explicitly.
+        extension. Set to 'fasta', 'fastq', 'sra-fastq', 'sam', or 'bam' to not
+        auto-detect. Colorspace is not auto-detected and must always be requested
+        explicitly.
 
     qualities -- When mode is 'w' and fileformat is None, this can be set to
         True or False to specify whether the written sequences will have quality
@@ -425,7 +517,10 @@ def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=N
           appropriately.
         * When False (no qualities available), an exception is raised when the
           auto-detected output format is FASTQ.
-
+    
+    single_input_read -- When file1 is a paired-end interleaved or SAM/BAM file,
+        this specifies whether to only use the first or second read (1 or 2) or
+        to use both reads (None).
     """
     if interleaved and (file2 is not None or qualfile is not None):
         raise ValueError("When interleaved is set, file2 and qualfile must be None")
@@ -434,10 +529,7 @@ def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=N
     
     if file2 is not None:
         return PairedSequenceReader(file1, file2, colorspace, fileformat)
-
-    if interleaved:
-        return InterleavedSequenceReader(file1, colorspace, fileformat)
-        
+    
     if qualfile is not None:
         if colorspace:
             # read from .(CS)FASTA/.QUAL
@@ -445,14 +537,11 @@ def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=N
         else:
             return FastaQualReader(file1, qualfile)
     
-    fastq_handler = ColorspaceFastqReader if colorspace else FastqReader
-    fasta_handler = ColorspaceFastaReader if colorspace else FastaReader
-
-    if fileformat is None and file1 != "-":
+    if fileformat is None and file1 != STDOUT:
         fileformat = guess_format_from_name(file1)
     
     if fileformat is None:
-        if file1 == "-":
+        if file1 == STDOUT:
             file1 = sys.stdin if mode == 'r' else sys.stdout
         for line in file1:
             if line.startswith('#'):
@@ -462,22 +551,45 @@ def open_reader(file1, file2=None, qualfile=None, colorspace=False, fileformat=N
                 fileformat = 'fasta'
             elif line.startswith('@'):
                 fileformat = 'fastq'
+            # TODO: guess SAM/BAM from data
             file1 = FileWithPrependedLine(file1, line)
             break
-        
-    fileformat = fileformat.lower()
-    if fileformat == 'fastq' and qualities is False:
-        raise ValueError("Output format cannot be FASTQ since no quality values are available.")
-    if fileformat == 'fasta':
-        return fasta_handler(file1)
-    if fileformat == 'fastq':
-        return fastq_handler(file1)
-    if fileformat == 'sra-fastq' and colorspace:
-        return SRAColorspaceFastqReader(file1)
+    
+    if fileformat is not None:
+        fileformat = fileformat.lower()
+        if fileformat in ("sam", "bam"):
+            if colorspace:
+                raise ValueError("SAM/BAM format is not currently supported for colorspace reads")
+            if interleaved:
+                return PairedEndSAMReader(file1)
+            elif single_input_read == 1:
+                return Read1SingleEndSAMReader(file1)
+            elif single_input_read == 2:
+                return Read2SingleEndSAMReader(file1)
+            else:
+                return SingleEndSAMReader(file1)
+        elif interleaved:
+            reader = InterleavedSequenceReader(file1, colorspace, fileformat)
+            if single_input_read == 1:
+                return paired_to_read1(reader)
+            elif single_input_read == 2:
+                return paired_to_read2(reader)
+            else:
+                return reader
+        elif fileformat == 'fasta':
+            fasta_handler = ColorspaceFastaReader if colorspace else FastaReader
+            return fasta_handler(file1)
+        elif fileformat == 'fastq':
+            if qualities is False:
+                raise ValueError("Output format cannot be FASTQ since no quality values are available.")
+            fastq_handler = ColorspaceFastqReader if colorspace else FastqReader
+            return fastq_handler(file1)
+        elif fileformat == 'sra-fastq' and colorspace:
+            return SRAColorspaceFastqReader(file1)
     
     raise UnknownFileType(
         "File format {0!r} is unknown (expected 'sra-fastq' (only for colorspace), "
-        "'fasta' or 'fastq').".format(fileformat))
+        "'fasta', 'fastq', 'sam', or 'bam').".format(fileformat))
 
 def guess_format_from_name(file, raise_on_failure=False):
     # Detect file format
@@ -494,6 +606,8 @@ def guess_format_from_name(file, raise_on_failure=False):
             return 'fasta'
         elif ext in ['.fastq', '.fq'] or (ext == '.txt' and name.endswith('_sequence')):
             return 'fastq'
+        elif ext in ('.sam', '.bam'):
+            return ext[1:]
     
     if raise_on_failure:
         raise UnknownFileType("Could not determine whether file {0!r} is FASTA "
@@ -784,7 +898,7 @@ class Writers(object):
     
     def close(self):
         for path in self.force_create:
-            if path not in self.writers and path != '-':
+            if path not in self.writers and path != STDOUT:
                 with open_output(path, "w"):
                     pass
         for writer in self.writers.values():
