@@ -1,6 +1,6 @@
 import math
 
-complement = {
+base_complements = {
     'A' : 'T',
     'C' : 'G',
     'R' : 'Y',
@@ -11,15 +11,25 @@ complement = {
     'D' : 'H',
     'N' : 'N'
 }
-for k,v in list(complement.items()):
-    complement[v] = k
-    complement[k.lower()] = v.lower()
-    complement[v.lower()] = k.lower()
+for k,v in list(base_complements.items()):
+    base_complements[v] = k
+    base_complements[k.lower()] = v.lower()
+    base_complements[v.lower()] = k.lower()
 
-def reverse_complement(seq):
-    return "".join(complement[base] for base in reversed(seq))
+MAGNITUDE = dict(
+    G=1E9,
+    M=1E6,
+    K=1E3
+)
 
 log2 = math.log(2)
+
+def complement(seq):
+    return "".join(base_complements[base] for base in seq)
+
+def reverse_complement(seq):
+    return "".join(base_complements[base] for base in reversed(seq))
+
 def sequence_complexity(seq):
     """A simple measure of sequence complexity"""
     seq = seq.upper()
@@ -32,39 +42,23 @@ def sequence_complexity(seq):
             term += d * math.log(d) / log2
     return -term
 
-MAGNITUDE = dict(
-    G=(1E9, "000000000"),
-    M=(1E6, "000000"),
-    K=(1E3, "000")
-)
+def qual2int(q, base=33):
+    """Convert a quality charater to a phred-scale int.
+    """
+    return ord(q) - base
 
-def magnitude_formatter(magnitude):
-    suffix = ""
-    if magnitude is None:
-        div = 1.0
-    else:
-        div = float(MAGNITUDE[magnitude.upper()][0])
-        suffix = magnitude
-    return lambda val: "{:.1f} {}".format(val / div, suffix)
-
-def int_or_str(x):
-    if x is None or isinstance(x, int):
-        return x
-    elif isinstance(x, str):
-        x = x.upper()
-        for a, mag in MAGNITUDE.items():
-            x = x.replace(a, mag[1])
-        return int(x)
-    else:
-        raise Exception("Unsupported type {}".format(x))
+def quals2ints(quals, base=33):
+    """Convert an iterable of quality characters to phred-scale ints.
+    """
+    return (ord(q) - base for q in quals)
 
 def enumerate_range(collection, start, end):
-        'Generates an indexed series:  (0,coll[0]), (1,coll[1]) ...'
-        i = start
-        it = iter(collection)
-        while i < end:
-            yield (i, next(it))
-            i += 1
+    'Generates an indexed series:  (0,coll[0]), (1,coll[1]) ...'
+    i = start
+    it = iter(collection)
+    while i < end:
+        yield (i, next(it))
+        i += 1
 
 def mean(data):
     return sum(data) / len(data)
