@@ -9,21 +9,25 @@ from .utils import run, files_equal, datapath, cutpath, redirect_stderr, tempora
 
 BACK_ALIGNERS = ('adapter', 'insert')
 
-def run_paired(params, in1, in2, expected1, expected2, aligners=('adapter',), callback=None):
+def run_paired(params, in1, in2, expected1, expected2, aligners=('adapter',),
+               callback=None, assert_files_equal=True):
     if type(params) is str:
         params = params.split()
     for aligner in aligners:
         with temporary_path('tmp1-' + expected1.format(aligner=aligner)) as p1:
             with temporary_path('tmp2-' + expected2.format(aligner=aligner)) as p2:
-                print(aligner)
                 p = params.copy()
                 p += ['--aligner', aligner, '-o', p1, '-p', p2]
-                p += ['-pe1', datapath(in1.format(aligner=aligner)), '-pe2', datapath(in2.format(aligner=aligner))]
-                assert atropos.main(p) is None
-                assert files_equal(cutpath(expected1.format(aligner=aligner)), p1)
-                assert files_equal(cutpath(expected2.format(aligner=aligner)), p2)
+                p += ['-pe1', datapath(in1.format(aligner=aligner)),
+                      '-pe2', datapath(in2.format(aligner=aligner))]
+                result = atropos.main(p)
+                assert isinstance(result, tuple)
+                assert len(result) == 3
+                if assert_files_equal:
+                    assert files_equal(cutpath(expected1.format(aligner=aligner)), p1)
+                    assert files_equal(cutpath(expected2.format(aligner=aligner)), p2)
                 if callback:
-                    callback(aligner)
+                    callback(aligner, p1, p2, result)
 
 def run_interleaved(params, inpath, expected, aligners=('adapter',)):
     if type(params) is str:
@@ -32,7 +36,9 @@ def run_interleaved(params, inpath, expected, aligners=('adapter',)):
         with temporary_path(expected.format(aligner=aligner)) as tmp:
             p = params.copy()
             p += ['--aligner', aligner, '-l', datapath(inpath.format(aligner=aligner)), '-L', tmp]
-            assert atropos.main(p) is None
+            result = atropos.main(p)
+            assert isinstance(result, tuple)
+            assert len(result) == 3
             assert files_equal(cutpath(expected.format(aligner=aligner)), tmp)
 
 # def run_interleaved2(params, inpath, expected1, expected2, aligners=('adapter',)):
@@ -67,7 +73,7 @@ def test_paired_end_legacy():
 def test_untrimmed_paired_output():
     with temporary_path("tmp-untrimmed.1.fastq") as untrimmed1:
         with temporary_path("tmp-untrimmed.2.fastq") as untrimmed2:
-            def callback(aligner):
+            def callback(aligner, p1, p2, result):
                 assert files_equal(cutpath('paired-untrimmed.1.fastq'), untrimmed1)
                 assert files_equal(cutpath('paired-untrimmed.2.fastq'), untrimmed2)
             run_paired(['-a', 'TTAGACATAT',
@@ -127,7 +133,6 @@ def test_second_too_short():
         with redirect_stderr():
             atropos.main('-a XX --paired-output out.fastq'.split() + [datapath('paired.1.fastq'), trunc2])
 
-@raises(SystemExit)
 def test_unmatched_read_names():
     with temporary_path("swapped.1.fastq") as swapped:
         try:
@@ -138,7 +143,12 @@ def test_unmatched_read_names():
             with open(swapped, 'w') as f:
                 f.writelines(lines)
             with redirect_stderr():
-                atropos.main('-a XX -o out1.fastq -p out2.fastq'.split() + ['-pe1', swapped, '-pe2', datapath('paired.2.fastq')])
+                result = atropos.main(
+                    '-a XX -o out1.fastq -p out2.fastq'.split() +
+                    ['-pe1', swapped, '-pe2', datapath('paired.2.fastq')])
+                assert isinstance(result, tuple)
+                assert len(result) == 3
+                assert result[0] != 0
         finally:
             os.remove('out1.fastq')
             os.remove('out2.fastq')
@@ -241,7 +251,7 @@ def test_pair_filter():
 def test_too_short_paired_output():
     with temporary_path("temp-too-short.1.fastq") as p1:
         with temporary_path("temp-too-short.2.fastq") as p2:
-            def callback(aligner):
+            def callback(aligner, _1, _2, result):
                 assert files_equal(cutpath('paired-too-short.1.fastq'), p1)
                 assert files_equal(cutpath('paired-too-short.2.fastq'), p2)
             run_paired('-a TTAGACATAT -A CAGTGGAGTA -m 14 --too-short-output '
@@ -254,7 +264,7 @@ def test_too_short_paired_output():
 def test_too_long_output():
     with temporary_path("temp-too-long.1.fastq") as p1:
         with temporary_path("temp-too-long.2.fastq") as p2:
-            def callback(aligner):
+            def callback(aligner, _1, _2, result):
                 assert files_equal(cutpath('paired_{aligner}.1.fastq'.format(aligner=aligner)), p1)
                 assert files_equal(cutpath('paired_{aligner}.2.fastq'.format(aligner=aligner)), p2)
             run_paired('-a TTAGACATAT -A CAGTGGAGTA -M 14 --too-long-output '
@@ -308,3 +318,22 @@ def test_no_insert_match():
 def test_overwrite():
     run_paired('-w 10,30,10', in1='lowq.fastq', in2='highq.fastq',
                expected1='lowq.fastq', expected2='highq.fastq')
+
+def test_no_writer_process():
+    def check_multifile(aligner, p1, p2, result):
+        assert os.path.basename(p1) == 'tmp1-out.1.fastq'
+        assert os.path.basename(p2) == 'tmp2-out.2.fastq'
+        tmpdir = os.path.dirname(p1)
+        assert tmpdir == os.path.dirname(p2)
+        assert os.path.exists(os.path.join(tmpdir, 'tmp1-out.1.0.fastq'))
+        assert os.path.exists(os.path.join(tmpdir, 'tmp1-out.1.1.fastq'))
+        assert os.path.exists(os.path.join(tmpdir, 'tmp2-out.2.0.fastq'))
+        assert os.path.exists(os.path.join(tmpdir, 'tmp2-out.2.1.fastq'))
+
+    run_paired('--threads 2 --no-writer-process --batch-size 1 -a AGATCGGAAGAGCACACGTCTGAACTCCAGTCACACAGTGATCTCGTATGCCGTCTTCTGCTTG" -A AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTCGGTGGTCGCCGTATCATT',
+        in1='big.1.fq', in2='big.2.fq',
+        expected1='out.1.fastq', expected2='out.2.fastq',
+        aligners=BACK_ALIGNERS, assert_files_equal=False,
+        callback=check_multifile
+    )
+    
